"""
Dictate - Push-to-Talk Voice Dictation for macOS

A local, privacy-first voice dictation menu bar app using Apple Silicon MLX models.
"""

__version__ = "2.2.0"
__author__ = "brando"

from dictate.config import Config

__all__ = ["Config", "__version__"]
